use xcb::render::Color;
use xcb::ffi::render::xcb_render_color_t;

macro_rules! color_const {
    ($name:ident, $r:expr, $g:expr, $b:expr, $a:expr) => {
        pub const $name: Color = Color {
            base: xcb_render_color_t {
                red: $r,
                green: $g,
                blue: $b,
                alpha: $a
            }
        };
    }
}

color_const!(WHITE, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF);
color_const!(BLACK, 0x0000, 0x0000, 0x0000, 0xFFFF);
color_const!(GREY, 0x6666, 0x6666, 0x6666, 0xFFFF);
color_const!(LIGHT_GREEN, 0x6666, 0xCCCC, 0x6666, 0xFFFF);
color_const!(RED, 0xCCCC, 0x0000, 0x3333, 0xFFFF);

fn double(num: u16) -> u16 {
    num << 8 | num
}

pub fn from_str(input: &str) -> Option<Color> {
    if input.len() != 7 {
        return None
    }
    let (head, body) = input.split_at(1);
    if head != "#" {
        return None
    }
    let valid = "0123456789ABCDEF";
    if !body.chars().all(|c| valid.contains(c)) {
        return None
    }
    let mut num = u32::from_str_radix(body, 16).unwrap();
    let blue = (num % 256) as u16;
    num >>= 8;
    let green = (num % 256) as u16;
    num >>= 8;
    let red = (num % 256) as u16;
    Some(Color::new(double(red), double(green), double(blue), 0xFFFF))
}

pub fn as_u32(input: Color) -> u32 {
    (input.alpha() as u32 % 256) << 24 |
        (input.red() as u32 % 256) << 16  |
        (input.green() as u32 % 256) << 16  |
        (input.blue() as u32 % 256)
}
