use xcb;

pub enum PictFormat {
    RGB24,
    ARGB32
}

fn format_rgb24(format: &xcb::render::Pictforminfo) -> bool {
    let depth = format.depth();
    let dformat = format.direct();
    depth == 24
        && dformat.red_mask() == 255
        && dformat.green_mask() == 255
        && dformat.blue_mask() == 255
        && dformat.red_shift() == 16
        && dformat.green_shift() == 8
        && dformat.blue_shift() == 0
}

fn format_argb32(format: &xcb::render::Pictforminfo) -> bool {
    let depth = format.depth();
    let dformat = format.direct();
    depth == 32
        && dformat.alpha_mask() == 255
        && dformat.red_mask() == 255
        && dformat.green_mask() == 255
        && dformat.blue_mask() == 255
        && dformat.alpha_shift() == 24
        && dformat.red_shift() == 16
        && dformat.green_shift() == 8
        && dformat.blue_shift() == 0
}

pub trait ConnectionExt {
    fn get_pict_format(&self, format: PictFormat) -> xcb::render::Pictformat;
}

impl ConnectionExt for xcb::Connection {
    fn get_pict_format(&self, format: PictFormat) -> xcb::render::Pictformat {
        let cookie = xcb::render::query_pict_formats(self);
        let reply = cookie.get_reply().unwrap();
        let spec: fn(&xcb::render::Pictforminfo) -> bool = match format {
            PictFormat::RGB24 => format_rgb24,
            PictFormat::ARGB32 => format_argb32
        };
        let format = reply.formats().find(spec).expect("Could not find PictFormat");
        format.id()
    }
}
