use xcb;

use std::mem;

#[repr(C)]
struct FontHeader {
    marker: u8,
    padding: [u8; 7],
    id: u32
}

#[repr(C)]
struct ELTHeader {
    len: u32,
    x: u16,
    y: u16
}

pub struct GlyphCmd {
    buf: Vec<u8>
}

impl GlyphCmd {
    pub fn new() -> Self {
        GlyphCmd { buf: vec![] }
    }

    pub fn set_glyphset(&mut self, id: xcb::render::Glyphset) {
        let header = FontHeader {
            marker: 0xFF,
            padding: [0; 7],
            id: id
        };
        let bytes: [u8; 12] = unsafe {
            mem::transmute(header)
        };
        self.buf.extend_from_slice(&bytes);
    }

    pub fn set_position(&mut self, x: u16, y: u16) {
        self.add_text_at("", x, y);
    }

    pub fn add_text(&mut self, string: &str) {
        self.add_text_at(string, 0, 0);
    }

    pub fn add_text_at(&mut self, string: &str, x: u16, y: u16) {
        let mut count = 0;
        let mut temp = vec![];
        for charcode in string.chars() {
            count += 1;
            let bytes: [u8; 4] = unsafe {
                mem::transmute(charcode)
            };
            temp.extend_from_slice(&bytes);
        }
        let header = ELTHeader {
            len: count,
            x: x,
            y: y
        };
        let bytes: [u8; 8] = unsafe {
            mem::transmute(header)
        };
        self.buf.extend_from_slice(&bytes);
        self.buf.extend_from_slice(&temp);
    }

    pub fn as_ref(&self) -> &[u8] {
        self.buf.as_slice()
    }
}

