use super::super::sensors;
use super::super::sensors::Sensor;
use ui::color;
use ui::context::Context;
use widgets::{Message, Update, Widget, WidgetParams};

const MARGIN: u16 = 7;

pub struct Sensors {
    context: Context,
    sensors: Vec<Box<Sensor>>
}

pub fn sensors(params: WidgetParams) -> Box<Widget> {
    let widget = Sensors {
        context: params.context,
        sensors: sensors::sensor_list(&params.config)
    };
    Box::new(widget)
}

impl Widget for Sensors {
    fn render(&mut self, x: u16, _w: u16) {
        let mut offset = x;
        for ref sensor in self.sensors.iter() {
            let icon_width = self.context.measure_text(&sensor.icon());
            let status_width = self.context.measure_text(&sensor.status());

            self.context.set_bg_color(color::GREY);
            self.context.draw_bg(offset, icon_width + MARGIN * 2);
            self.context.draw_text(&sensor.icon(), offset + MARGIN);
            offset += icon_width + MARGIN * 2;

            self.context.set_bg_color(color::BLACK);
            self.context.draw_text(&sensor.status(), offset + MARGIN);
            offset += status_width + MARGIN * 2;
        }
    }

    fn width(&mut self) -> u16 {
        let mut sum = 0;
        for ref sensor in self.sensors.iter() {
            let text = format!("{}{}", sensor.icon(), sensor.status());
            sum += self.context.measure_text(&text) + MARGIN * 4;
        }
        sum
    }

    fn handle_event(&mut self, event: &Message) -> Update {
        match event {
            &Message::Update => {
                for ref mut sensor in self.sensors.iter_mut() {
                    sensor.process()
                }
                Update::Relayout
            },
            _ => Update::Nothing
        }
    }
}
