use xcb;

use ui::context::Context;
use ui::x11;
use widgets::{Message, Update, Widget, WidgetParams};

const MARGIN: u16 = 7;

pub struct Title {
    conn: x11::Connection,
    context: Context,
    title: String,
    last_win: xcb::Window
}

pub fn title(params: WidgetParams) -> Box<Widget> {
    let widget = Title {
        conn: params.conn(),
        context: params.context,
        title: "".to_string(),
        last_win: 0
    };
    Box::new(widget)
}

impl Widget for Title {
    fn init(&mut self) {
        let screen = self.conn.default_screen();
        self.last_win = screen.get_active_window();
        self.title = self.conn.get_window_name(self.last_win);
        self.conn.watch(screen.root, true);
        self.conn.watch(self.last_win, true);
    }

    fn render(&mut self, x: u16, w: u16) {
        let width = w.saturating_sub(MARGIN * 2);
        self.context.draw_text_with_clipping(&self.title, x + MARGIN, width);
    }

    fn width(&mut self) -> u16 {
        0
    }

    fn fit_width(&self) -> bool {
        true
    }

    fn handle_event(&mut self, event: &Message) -> Update {
        match event {
            &Message::XcbEvent(ref event) =>
                if event.response_type() == xcb::PROPERTY_NOTIFY {
                    let event: &xcb::PropertyNotifyEvent = unsafe { xcb::cast_event(&event) };
                    if event.atom() == self.conn.atom(x11::_NET_ACTIVE_WINDOW) {
                        let new_win = {
                            let screen = self.conn.default_screen();
                            screen.get_active_window()
                        };
                        self.conn.watch(self.last_win, false);
                        self.conn.watch(new_win, true);
                        self.last_win = new_win;
                        self.title = self.conn.get_window_name(new_win);
                        Update::Rerender
                    }
                    else if event.atom() == self.conn.atom(x11::_NET_WM_NAME) {
                        self.title = self.conn.get_window_name(self.last_win);
                        Update::Rerender
                    }
                    else {
                        Update::Nothing
                    }
                }
                else {
                    Update::Nothing
                },
            _ => Update::Nothing
        }
    }

    fn finish(&mut self) {
        let screen = self.conn.default_screen();
        self.conn.watch(screen.root, false);
        self.conn.watch(self.last_win, false);
    }
}
